<?php

namespace Btravel\Category\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Category\Http\Requests\BtCategoryRequest;
use Btravel\Category\Interfaces\BtCategoryRepositoryInterface;
use Btravel\Category\Models\BtCategory;

/**
 * Resource controller class for bt_category.
 */
class BtCategoryResourceController extends BaseController
{

    /**
     * Initialize bt_category resource controller.
     *
     * @param type BtCategoryRepositoryInterface $bt_category
     *
     * @return null
     */
    public function __construct(BtCategoryRepositoryInterface $bt_category)
    {
        parent::__construct();
        $this->repository = $bt_category;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Category\Repositories\Criteria\BtCategoryResourceCriteria::class);
    }

    /**
     * Display a list of bt_category.
     *
     * @return Response
     */
    public function index(BtCategoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Category\Repositories\Presenter\BtCategoryPresenter::class)
                ->$function();
        }

        $bt_categories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('category::bt_category.names'))
            ->view('category::bt_category.index', true)
            ->data(compact('bt_categories', 'view'))
            ->output();
    }

    /**
     * Display bt_category.
     *
     * @param Request $request
     * @param Model   $bt_category
     *
     * @return Response
     */
    public function show(BtCategoryRequest $request, BtCategory $bt_category)
    {

        if ($bt_category->exists) {
            $view = 'category::bt_category.show';
        } else {
            $view = 'category::bt_category.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('category::bt_category.name'))
            ->data(compact('bt_category'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new bt_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BtCategoryRequest $request)
    {

        $bt_category = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('category::bt_category.name')) 
            ->view('category::bt_category.create', true) 
            ->data(compact('bt_category'))
            ->output();
    }

    /**
     * Create new bt_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BtCategoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $bt_category                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('category::bt_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('category/bt_category/' . $bt_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/category/bt_category'))
                ->redirect();
        }

    }

    /**
     * Show bt_category for editing.
     *
     * @param Request $request
     * @param Model   $bt_category
     *
     * @return Response
     */
    public function edit(BtCategoryRequest $request, BtCategory $bt_category)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('category::bt_category.name'))
            ->view('category::bt_category.edit', true)
            ->data(compact('bt_category'))
            ->output();
    }

    /**
     * Update the bt_category.
     *
     * @param Request $request
     * @param Model   $bt_category
     *
     * @return Response
     */
    public function update(BtCategoryRequest $request, BtCategory $bt_category)
    {
        try {
            $attributes = $request->all();

            $bt_category->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('category::bt_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('category/bt_category/' . $bt_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('category/bt_category/' . $bt_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the bt_category.
     *
     * @param Model   $bt_category
     *
     * @return Response
     */
    public function destroy(BtCategoryRequest $request, BtCategory $bt_category)
    {
        try {

            $bt_category->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('category::bt_category.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('category/bt_category/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('category/bt_category/' . $bt_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple bt_category.
     *
     * @param Model   $bt_category
     *
     * @return Response
     */
    public function delete(BtCategoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('category::bt_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('category/bt_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/category/bt_category'))
                ->redirect();
        }

    }

    /**
     * Restore deleted bt_categories.
     *
     * @param Model   $bt_category
     *
     * @return Response
     */
    public function restore(BtCategoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('category::bt_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/category/bt_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/category/bt_category/'))
                ->redirect();
        }

    }

}
